#!/bin/bash 

function force_exit {
    echo "Committing suicide (PID $$)"
    exit
}

trap force_exit SIGINT

. ./config.sh

# Provides a list of good hosts (that contains QDR connection)
function hosts() {
    num="$1"
    hlist=${NODES[0]}
    for POS in $(seq 1 $(($num - 1))) ; do 
        hlist="$hlist,${NODES[$POS]}"
    done
    echo $hlist
}


for COUNT in ${COUNT_ARR[@]}; do
for TYPE in ${TYPE_ARR[@]}; do
for NN in ${NN_ARR[@]}; do 
for T in ${T_ARR[@]}; do 
for PPN in ${PPN_ARR[@]}; do 
for API in ${API_ARR[@]}; do 
    BENCHFILE="./output_v2/COUNT:$COUNT-NN:$NN-PPN:$PPN-API:$API-T:$T-TYPE:$TYPE.txt"

    if [ ! -e "${BENCHFILE}" ]; then
        OUTDIR="$(dirname $BENCHFILE)"
        [ ! -d $OUTDIR ] && mkdir $OUTDIR
        touch $BENCHFILE

        if [[ "read" == $TYPE ]]; then
            IOR_TYPE_OPTS="-r -E -k"
            LUSTRE_TESTFILE="$TD/sharedread$NN/file"
            TESTDIR="$(dirname $LUSTRE_TESTFILE)"
            lfs getstripe $TESTDIR | tee -a $BENCHFILE
        elif [[ "write" == $TYPE ]]; then
            IOR_TYPE_OPTS="-w"
            LUSTRE_TESTFILE="$TD/sharedwrite$NN/file"
            TESTDIR="$(dirname $LUSTRE_TESTFILE)"
            [ -d $TESTDIR ] && rm -r $TESTDIR 
            mkdir -p $TESTDIR 
            lfs setstripe	-c $((2 * $NN)) $TESTDIR
            lfs getstripe $TESTDIR | tee -a $BENCHFILE
        fi

        IOR_PARAMS="-i 1 "
        IOR_PARAMS+="-s 1 -t $T -b $(($DATASIZE / $PPN)) "
        IOR_PARAMS+="-D 60 -O stoneWallingWearOut=1 "
        IOR_PARAMS+="-a $API "
        #IOR_PARAMS+="-p " # preallocate -- preallocate file size
        #IOR_PARAMS+="-e " # fsync -- perform sync operation after each block write
        IOR_PARAMS+="-g " # intraTestBarriers -- use barriers between open, write/read, and close
        IOR_PARAMS+="-H " # showHints -- show hints 
        IOR_PARAMS+="-z " # randomOffset -- access is to random, not sequential, offsets within a file
        IOR_PARAMS+="-o $LUSTRE_TESTFILE $IOR_TYPE_OPTS "
        IOR_PARAMS+="-O summaryFile=${BENCHFILE}.json "
        IOR_PARAMS+="-O summaryFormat=JSON "
        
        if [[ "isc17" == ${hostname:0:5} ]]; then
            $MPIEXEC -np $NN --hosts $(hosts $NN) ./drop_caches.sh
            MPIEXEC_PARAMS=" -ppn $PPN -np $(($NN * $PPN)) --hosts $(hosts $NN) -wdir $WD "
            MPIEXEC_PARAMS+="-genv MV2_NUM_HCAS 1 "
            MPIEXEC_PARAMS+="-genv MV2_CPU_BINDING_LEVEL core "
            MPIEXEC_PARAMS+="-genv MV2_CPU_BINDING_POLICY scatter "
        elif [[ "m" == ${hostname:0:1} ]]; then
            #MPIEXEC_PARAMS=" -ppn $PPN -n $(($NN * $PPN)) -wdir $WD --host $(hosts $NN) "
            MPIEXEC_PARAMS=" -ppn $PPN -n $(($NN * $PPN)) -wdir $WD "
        else
            echo "Cluster $hostname is not supported. Quitting."
            exit 1
        fi

        NETOUTDIR="${BENCHFILE}_network"
        [[ ! -d $NETOUTDIR ]] && mkdir $NETOUTDIR || rm $NETOUTDIR/HOST*PID*.txt

        (
        set -x
        $MPIEXEC $MPIEXEC_PARAMS ./ior_wrapper.sh "$IOR_PARAMS" "$NETOUTDIR" | tee -a $BENCHFILE
        #$MPIEXEC $MPIEXEC_PARAMS $IOR $IOR_PARAMS | tee -a $BENCHFILE
        set +x
        ) 2> >(tee -a $BENCHFILE)
    else
        echo "skip $(readlink -f $BENCHFILE), already exists"
    fi

done
done
done
done
done
done

